function [V,Y,Lo,Hi,iter,fcount] = mynelder(F,V,xtol,maxiter)
% function [V,Y,Lo,Hi,iter,fcount] = mynelder(F,V,xtol,maxiter)
%------------------------------------------------------------------------------
%*** purpose:
%	Nelder-Mead minimization method
%*** parameters:
%	F		-in-	fcion name F(P), objective fcion (return scalar)
%	V(1:m,1:n+1)	-in-	V(:,i) param.values for vertices (i=1:n+1) 
%	xtol		-in-	precision bound on variation of cost
%	maxiter		-in-	max iters
%	V(1:m,1:n+1)	-out-	V(:,i) param.values for vertices (i=1:n+1) 
%	Y(1:n+1)	-out-	cost fcion value for vertex i=1:n+1
%	Lo,Hi		-out-	integer ptrs to V(:,Lo),V(:,Hi) (min/max vertex)
%	iter		-out-	actual iterations iters
%	fcount		-out-	no. of fcion evaluations
%------------------------------------------------------------------------------
	freq = 1;
	fcount = 0;
	iter=0;
        fnamecost = 'cost.dat';
        fnamebest = 'best.dat';
        fnamesplx = 'splx.dat';
%*** open files:
        fidcost = fopen(fnamecost,'wt');
	fidbest = fopen(fnamebest,'wt');
%*** init simplex:
	n = size(V,2)-1;
        n1 = size(V,1);
        Y = zeros(n+1,1);
	for j=1:n+1
	  Y(j) = feval(F,V(:,j));  fcount = fcount + 1;
	  fprintf('mynelder: init %3d / %3d   Y: %e\n',j,n+1,Y(j));
	end;
%*** write the whole simplex:
	fidsplx = fopen(fnamesplx,'wt');
	fmt = repmat('%e ',1,n1);
	fmt = strcat(fmt,' \n');
        for j=1:n+1
	  fprintf(fidsplx,fmt,V(:,j));
        end;
	fclose(fidsplx);
%*** find vertices with highest and lowest cost and second highest/lowest:
        [w,p] = sort(Y);
        Hi = p(n+1);
        Lo = p(1);
        Li = p(2);
        Ho = p(n);
	Lo0 = Lo;
	bestfit = Y(Lo);
%*** determine the max. size of the simplex:
	sizeV = max(sqrt(sum((V-V(:,Lo)*ones(1,n+1)).^2,1)));

%*** writeout:
	fprintf('it:%4d  sizeV: %e  Y: %e  %e  |dY|: %e\n',...
               iter, sizeV, Y(Hi), Y(Lo),Y(Hi)-Y(Lo));

%=== main loop starts here: form the new points:
	best = 0;
	for iter = 1:maxiter
%*** if the tolerance is attained, quit:
	  if (Y(Hi) <= (Y(Lo)+xtol)) 
            break;
          end;
%*** otherwise, replace the worst vertex:
	  M = (sum(V,2) - V(:,Hi))/n;
          R = 2*M - V(:,Hi);
          YR = feval(F,R); fcount = fcount + 1;
	  if (YR < bestfit)  
	    foundBest(R);
	  end;
%*** improve the simplex: replace a vertex:
 	  if (YR < Y(Ho))
	    if (Y(Li) < YR) 
	      V(:,Hi) = R;
              Y(Hi) = YR;
	    else
	      E = 2*R-M;
              YE = feval(F,E); fcount = fcount + 1;
	      if (YE < bestfit)  
	        foundBest(E);
	      end;
              if (YE < Y(Li)) 
	        V(:,Hi) = E;
                Y(Hi) = YE;
	      else
	        V(:,Hi) = R;
                Y(Hi) = YR;
              end;
            end;
          else
            if (YR < Y(Hi)) 
	      V(:,Hi) = R;
              Y(Hi) = YR;
            end;
	    C = (V(:,Hi)+M)/2;
            YC = feval(F,C); fcount = fcount + 1;
	    if (YC < bestfit) 
	      foundBest(C);
	    end;
            if (YC < Y(Hi))
	      V(:,Hi) = C;
              Y(Hi) = YC;
            else
%*** shrink the simplex:
	      for j = 1:n+1
	        if (j~=Lo) 
	          V(:,j) = (V(:,j)+V(:,Lo))/2;
                  Y(j) = feval(F,V(:,j)); fcount = fcount + 1;
	          if (Y(j) < bestfit)  
	 	    foundBest(V(:,j));
	          end;
                end;
              end;
            end;
          end;
%*** find vertex with highest and lowest cost and second hi/lowest cost:
          [w,p] = sort(Y);
          Hi = p(n+1);
          Lo = p(1);
          Li = p(2);
          Ho = p(n);
%*** determine the max. size of the simplex:
	  sizeV = max(sqrt(sum((V-V(:,Lo)*ones(1,n+1)).^2,1)));

%*** writeout:
	  fprintf('it:%4d  sizeV: %e  Y: %e  %e  |dY|: %e\n',...
                  iter, sizeV, Y(Hi), Y(Lo),Y(Hi)-Y(Lo));

%*** save cost fcions:
	  if (fidcost~=0)
	    fmt = repmat('%e ',1,n+1);
	    fmt = strcat(fmt,' \n');
	    fprintf(fidcost,fmt,Y);
	  end;

%*** savesolution if best one occured:
	  if (Lo0~=Lo)
	    best = 1;
	    Lo0 = Lo;
	    bestfit = Y(Lo);
	  end;
	  if ((best~=0) && (rem(iter,freq)==0))
	    best = 0;
%	    fprintf('=== mynelder: writeout: resolve for best one\n');
	    aux1 = feval(F,V(:,Lo)); fcount = fcount + 1;
	    foundBest(V(:,Lo));
	    fmt = repmat('%20.15e ',1,size(V,1));
	    fmt = strcat(fmt,' \n');
%*** write the best:
	    fprintf(fidbest,fmt,V(:,Lo));
          end;
          if 1
%*** write the whole simplex:
	    fmt = repmat('%20.15e ',1,size(V,1));
	    fmt = strcat(fmt,' \n');
	    fidsplx = fopen(fnamesplx,'wt');
            for j=1:n+1
	      fprintf(fidsplx,fmt,V(:,j));
            end;
	    fclose(fidsplx);
	  end;
	end;
%=== end of the main loop
	iter=iter-1;
%*** close files:
	fclose(fidcost);
	fclose(fidbest);
	return;


%----------------------------------------------------------------------------
% subfunction foundBest:
%----------------------------------------------------------------------------
function foundBest(V);
% function foundBest(V);
% saves the best V:

return;

