function [dAux,dl] = transport2Dnew(cuml,luxDN,t,Aux,l,Perm,param)
% function [dAux,dl] = transport(t,Aux,l)
% assembles ODE of the model elongation of hypocotyl (1D)
% model Grieneissen
%3.8.3: Add remainFR constraint to 0.1 to 0.9

%*** parameters:
% t			-in-	time 
% Aux=A		-in-	Auxin in each cell at time (t), CONENTRATION
% l        	-in-	length of each cell at time t (width = 1, is constant
% over time)
% D         -in-
% D=[Detop,Debottom,Deright,Deleft,Dstop,Dsbottom,Dsright,Dsleft]
% each term is a matrix of same dimension that Aux
% dAux		-out-	time derivatives of Auxin
% dl		-out-	time derivatives of the length
%

%the first cell is the bottom cell

global DATA
setdata(param)
%initdata

%*** init sizes:
 nrow=size(Aux,1);

%*** init state vectors:
  A      = Aux;          %concentration of auxin in cells

%*** permeability matrices
  Dtop=Perm(:,:,1);
  Dbottom=Perm(:,:,2);
  Dout=Perm(:,:,3);
  Din=Perm(:,:,4);

  
%*** prepare indices for rows
  i = 2:(nrow-1);
  iminus = 1:(nrow-2);
  iplus = 3:nrow;
  
%*** prepare indices for columns j=1=endoderm, j=4=epiderm
  
  
%*** derivative of the length
%   dl = zeros(nrow,4);
%   dlsplus = zeros(nrow,1);
%   dl_light = zeros(nrow,1);

  dlsplus=splus(DATA.Asens,DATA.mudecay,A(:,4)).*DATA.growthfact; 
  dli=dlsplus;%.*dl_light;
  dl=[dli,dli,dli,dli];
    
%*** prepare permeability term i=1-> bottom cell
  pin_basal=Dbottom.*A.*l;
  pin_antibasal=Dtop.*A.*l;
  pin_int=Din.*A.*l;
  pin_ext=Dout.*A.*l;
  
  pout_antibasal=Dtop.*A;
  pout_basal=Dbottom.*A;
  pout_ext=Dout.*A;
  pout_int=Din.*A;
  
  

  Diff=zeros(nrow,4);
  % bottom cells !!!! vers.2.6 add: + pin_antibasal (REMOVED)
  Diff(1,1)= pin_basal(2,1)/l(1,1) + pin_int(1,2)/l(1,1)...
             -(pout_antibasal(1,1)+pout_ext(1,1));
            %endoderm 
  Diff(1,2:3)= 1./l(1,2:3).*(pin_basal(2,2:3)+pin_ext(1,1:2) + pin_int(1,3:4) )...
              - (pout_antibasal(1,2:3)+pout_ext(1,2:3)+pout_int(1,2:3));
               % cortex
  Diff(1,4) = 1/l(1,4)*(pin_basal(2,4)+pin_ext(1,3)) ...
              -(pout_antibasal(1,4)+pout_int(1,4));
             %epiderm   
            
  % middle cells of endoderm
  Diff(i,1)=1./l(i,1).*(pin_basal(iplus,1)+pin_antibasal(iminus,1)+pin_int(i,2)) ...
         -(pout_antibasal(i,1)+pout_basal(i,1)+pout_ext(i,1));
           
     
   % middle cells of epiderm
  Diff(i,4) = 1./l(i,4).*(pin_basal(iplus,4)+pin_antibasal(iminus,4)+pin_ext(i,3)) ...
         -(pout_basal(i,4)+pout_antibasal(i,4)+pout_int(i,4));
     
  %middle cells of cortex
  Diff(i,2:3)=1./l(i,2:3).*(pin_basal(iplus,2:3)+pin_antibasal(iminus,2:3)+pin_ext(i,1:2)+pin_int(i,3:4))...
           -(pout_antibasal(i,2:3)+pout_basal(i,2:3)+pout_ext(i,2:3)+pout_int(i,2:3));
             
   %top cells         
   Diff(nrow,1)= 1./l(nrow,1).*(pin_antibasal(nrow-1,1)+pin_int(nrow,2)) ...
			- (pout_basal(nrow,1)+pout_ext(nrow,1));
			 %+ auxbios(t,cuml,luxDN);%endoderm
   Diff(nrow,2:3)= 1./l(nrow,2:3).*(pin_antibasal((nrow-1),2:3)+pin_ext(nrow,1:2) + pin_int(nrow,3:4)) ...
              - (pout_basal(nrow,2:3)+pout_ext(nrow,2:3)+pout_int(nrow,2:3));
          % cortex
   Diff(nrow,4) = 1./l(nrow,4).*(pin_antibasal(nrow-1,4)+pin_ext(nrow,3)) ...
              -(pout_basal(nrow,4)+pout_int(nrow,4));
              %epiderm     

  
 

%*** derivative of the length
   %dAux = zeros(nrow,4);
   dAux = Diff -A.*dl./l;
   dAux(nrow,1)=dAux(nrow,1)+1*auxbios(t,cuml,luxDN);
   %dAux(nrow,4)=dAux(nrow,4)+0*auxbios(t,cuml,luxDN); %epidermis
   dAux(1,:)=dAux(1,:)-Dbottom(1,:).*A(1,:);
  
  
% %*** assing results to dAux :
%   dAux = zeros(ncell,1);
   dAux = dAux- DATA.nu.*A./l.*splus(DATA.Klight,DATA.Rlight,luxDN); %modif nov 23 /l

  
  
%*** define the function M
function y=M(param,u,x)

y=(x-u)./(param+x-u);

%*** define the function S
function y=splus(a,u,x)

y=((x-u).^2)./(a+(x-u).^2).*J(u,x);

%*** define the function h for length reaction to the light
function y=h(a,u,x)
y=((x-u).^2)./(a+(x-u).^3).*J(u,x);


%*** define the function J (indicatrice)
function y=J(a,x)
  y=zeros(size(x));
  %y(x>a)=1;
  if ((x>a))  
      y=1.0; 
   else y=0.0;
  end

%*** define the function Spoisson
% function y=spoisson(a,u,x)

%y=((x-u))./(a+(x-u).^2);


%*** define the function alpha (effect of FR, u=R:FR ratio)
function y=alpha(t,light)
global DATA

	statlight=light/DATA.lux;
if t>DATA.delayFR
	at=mod((t-16)/8,1); %only for LD
	if (statlight>0.5) %==1) %day @Flo changed
	u=DATA.nRFR/(DATA.uFR);
	else %night
	if at< - log2(DATA.remainFR)/2 %0.3;
		ufr = DATA.uFR*2^(-2*at);
	else
	ufr=DATA.uFR*DATA.remainFR; %0.3;	
	end
	u=DATA.nRFR/ufr;
	end
else
	if (statlight>0.5)
	u=DATA.nRFR; %*light/DATA.lux;
	else 
	u=DATA.nRFR/DATA.remainFR; 
	end
end
y= (1-splus(DATA.ratioRFR,DATA.decayRFR,u));

%*** define auxin biosynthesis
function y=auxbios(t,cuml,light)
global DATA
y=DATA.mu*alpha(t,light)*M(DATA.Mtcumul,DATA.Mtdelay,cuml); %*(1-M(DATA.Rlight,light));

